/*
Copyright (c) 2008, Christopher J. Creevey

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

void random_star_decom(char *tree, int number_of_taxa);

int main(int argc, char *argv[])
	{
	int number = 0;
	char *result = '\0';
	
	srand((unsigned) (time(NULL)/2)+getpid());
	
	
	if(argc < 2)
		{
		printf("random_tree:\n\tThis program, creates a random tree of size specified by the user\n\tIt uses a markovian method to produce the trees\n\n\tUsage:  random_trees <number of taxa>\n\n");
		exit(1);
		}
	
	number = atoi(argv[1]);
	if(number < 4)
		{
		printf("ERROR: You must pass a number greater than 3 to this software");
		exit(1);
		}
	
	result = malloc(90000000*sizeof(char));
	result[0] = '\0';
	random_star_decom(result, number);
	printf("%s\n", result);
	free(result);
	
	
	}


void random_star_decom(char *tree, int number_of_taxa)
    {
    char **treearray = '\0', *tempchar = '\0', temp[1000000];
    int i=0, j=0, k=0, l=0, nodes = number_of_taxa, random1 = 0, random2 = 0;
	float distance = 0;
    
    tempchar = malloc(9000000*sizeof(char));
    tempchar[0] = '\0';
	temp[0] = '\0';
    
	treearray = malloc(number_of_taxa*sizeof(char *));
    if(!treearray) exit(0);
    for(i=0; i<number_of_taxa; i++)
        {
        treearray[i] = malloc(9000000*sizeof(char));
        if(!treearray[i]) exit(0);
        treearray[i][0] = '\0';
        }
    
     /** copy the names of the taxa into the array **/
    
    for(i=0; i<number_of_taxa; i++)
        {
		sprintf(temp, "%d", i);
        strcpy(treearray[i], temp);
        }
	
	
    /** at each round, combine two of the nodes into one node **/
    
    while(nodes > 1)
        {
        
        /** randomly pick the two nodes to be combined **/
        random1 = (int)fmod(rand(), nodes);
        random2 = (int)fmod(rand(), nodes);
        while(random2 == random1)
            {
            random2 = (int)fmod(rand(), nodes);
            }
        /** these numbers represent the ith numbers in turn left in the array **/
        i=-1;k=0;
        do
            {
            if(strcmp(treearray[k], "") != 0 )
                {
                i++;
                }
            k++;
            }while(i != random1 && k < number_of_taxa);
        k--;
        j=-1;l=0;
        do
            {
            if(strcmp(treearray[l], "") != 0 )
                {
                j++;
                }
            l++;
            }while(j != random2 && k < number_of_taxa);
        l--;
        /** combine the contents of the two nodes **/
        
        strcpy(tempchar, "( ");
        strcat(tempchar, treearray[k]);
		random1 = (int)fmod(rand(), 100);
		distance = (float)random1/(float)100;
		sprintf(temp, " :%.2f", distance);
		strcat(tempchar, temp);
		
        strcat(tempchar, ", ");
		
        strcat(tempchar, treearray[l]);
		random1 = (int)fmod(rand(), 100);
		distance = (float)random1/(float)100;
		sprintf(temp, " :%.2f", distance);
		strcat(tempchar, temp);
		
        strcat(tempchar, ")");
		
        strcpy(treearray[k], "");
        strcpy(treearray[l], tempchar);
        
        
        nodes--;
        }
    
    /** find the position of the completed tree and copy it into the string passed to the function **/
    i=0;
    while(strcmp(treearray[i], "") == 0 && i < number_of_taxa)
        i++;
            
    strcpy(tree, treearray[i]);
    strcat(tree, ";");
    
    
    /** free up allocated memory **/
    
    for(i=0; i<number_of_taxa; i++)
        {
        free(treearray[i]);
        treearray[i] = '\0';
        }
    free(treearray);
    treearray = '\0';
    free(tempchar);
    tempchar = '\0';
    
    }
    
