#!/usr/bin/env python
"""
FUNCTION: Reads a clustblast file and adds the other hits that are
          members of clusters of which only representatives are in the
          database so far. It also needs the fasta file that contain the
          hit protein sequences.

USAGE: PKS_db_add_cluster_members.py <DB_FILE> <BLASTCLUST_FILE> <FASTA_WITH_HMM_HIT_FEATURES> <DB_SWITCH>

       <DB_SWITCH> can be
         - 'insert' to put new data into the database
         - 'update' to change entries in the database

Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys
import fastaparser

import pks_db_tools
import PKS_db_add_hmm_hit_properties

def main():
    if not len(sys.argv) == 5  or (not sys.argv[4] in ['insert', 'update']):
        print __doc__
        sys.exit(2)
    cluster_adder = ClusterAdder(sys.argv[1])
    cluster_adder.read_blastclust_file(sys.argv[2])
    cluster_adder.get_annotation_from_hit_seq_fasta(sys.argv[3])
    cluster_adder.add_cluster_members_to_db(sys.argv[4])

class ClusterAdder(object):

    clusters = []
    hits_and_hits_features = {}

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        self.parser = fastaparser.FastaParser()
        self.HHM_fasta_toolbox = pks_db_tools.HMMFastaToolBox()

    def read_blastclust_file(self, input_blastclust_file):
        for line in open(input_blastclust_file):
            if line[0] in ['\n', '#']: continue
            cluster = line[:-1].split()
            if len(cluster) == 1: continue
            self.clusters.append(cluster)

    def get_annotation_from_hit_seq_fasta(self, input_fasta_file):
        """Read fasta file that contains the HMM hit sequences"""
        parser = fastaparser.FastaParser()
        for header, seq in parser.parse_fasta_file(input_fasta_file):
            short_header = header.split()[0]
            self.hits_and_hits_features[short_header] = header

    def add_cluster_members_to_db(self, insert_update_switch):
        self.db_connector.create_db_connection()
        for cluster in self.clusters:
            if self.__check_if_representativ_in_db(cluster[0]) == False:
                print 'There is no represenative from the following cluster ' \
                      'in the database. Maybe it was removed during the tree ' \
                      'cleaing:\n%s' % (cluster)
                continue
            cluster_representative = cluster[0]
            for member in cluster[1:]:
                hit_features = self.HHM_fasta_toolbox.get_hit_features(
                    self.hits_and_hits_features[member])
                if insert_update_switch == 'insert':
                    self.db_connector.fill_hit_table(
                        hit_features, '-', cluster_representative)
                elif insert_update_switch == 'update':
                    self.db_connector.update_hit_table(
                        hit_features, '-', cluster_representative)
        self.db_connector.close_db_connection()

    def __check_if_representativ_in_db(self, first_cluster_member):
        HMM_domain_name = self.HHM_fasta_toolbox.get_hit_features(
                self.hits_and_hits_features[first_cluster_member])[
            'HMM_domain_name']
        self.db_connector.create_db_connection()
        cur = self.db_connector.con.cursor()
        cur.execute('SELECT domain_HMM_hit_name FROM HMM_hits WHERE '
                    'domain_HMM_hit_name="%s_%s";' % (
                    HMM_domain_name, first_cluster_member))
        if cur.fetchone() == None:
            return (False)
        else:
            return(True)
        self.db_connector.close_db_connection()

if __name__ == '__main__': main()
