#!/usr/bin/env python
"""
FUNCTION: Adds the features of HMM hits to the database and reads the
          replacement table produced by simplify_fasta_headers.py for
          doing this.

USAGE: PKS_db_add_hmm_hit_properties.py <DB_FILE> <REPLACEMENT_TABEL_FILE>

Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import pks_db_tools

def main():
    if not len(sys.argv) == 3:
        print __doc__
        sys.exit(2)
    HMM_hit_adder = HMMHitAdder(sys.argv[1])
    HMM_hit_adder.read_replacement_table_file(sys.argv[2])

class HMMHitAdder(object):

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            self.db_connector.create_db_file()

    def read_replacement_table_file(self, replacement_table_file):
        self.db_connector.create_db_connection()
        for line in open(replacement_table_file):
            if line[0] in ['\n', '#']: continue
            hit_id, full_header = line.split('\t')
            HMM_fasta_toolbox = pks_db_tools.HMMFastaToolBox()
            hit_features = HMM_fasta_toolbox.get_hit_features(full_header)
            self.db_connector.fill_hit_table(hit_features, hit_id, '-')
        self.db_connector.close_db_connection()

if __name__ == '__main__': main()
