#!/usr/bin/env python
"""
FUNCTION: Fixes some minor species issues.

USAGE: PKS_db_add_manually_fixed_species_properties.py <DB_FILE> <DB_SWITCH>

       <DB_SWITCH> can be
         - 'insert' to put new data into the database
         - 'update' to change entries in the database

Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys
import pks_db_tools

def main():
    if not len(sys.argv) == 3 or (not sys.argv[2] in ['insert', 'update']):
        print __doc__
        sys.exit(2)
    adder = SpeciesPropertyAdder(sys.argv[1])
    adder.add_manually_fixed_tax_strings(sys.argv[2])

class SpeciesPropertyAdder(object):

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.db_connector.create_db_connection()

    def add_manually_fixed_tax_strings(self, insert_update_switch):
        self._set_species_string_combos()
        for species, taxonomy_string in self.species_and_tax_strings:
            if insert_update_switch == 'insert':
                self.db_connector.fill_species_table(species, taxonomy_string)
            elif insert_update_switch == 'update':
                self.db_connector.update_species_table(species, taxonomy_string)

    def _set_species_string_combos(self):
        self.species_and_tax_strings = [
            ['Blochmannia floridanus', 'root|cellular organisms|Bacteria|Proteobacteria|Gammaproteobacteria|Enterobacteriales|Enterobacteriaceae'],
            ['Blochmannia pennsylvanicus', 'root|cellular organisms|Bacteria|Proteobacteria|G\
            ammaproteobacteria|Enterobacteriales|Enterobacteriaceae'],
            ['Bos Taurus', 'root|cellular organisms|Eukaryota|Fungi/Metazoa group|Metazoa|Eumetazoa|Bilateria|Coelomata|Deuterostomia|Chordata|Craniata|Vertebrata|Gnathostomata|Teleostomi|Euteleostomi|Sarcopterygii|Tetrapoda|Amniota|Mammalia|Theria|Eutheria|Laurasiatheria|Cetartiodactyla|Ruminantia|Pecora|Bovidae|Bovinae|Bos|Bos taurus'],
            ['Cephalosporium acremonium', 'root|cellular organisms|Eukaryota|Fungi/Metazoa group|Fungi|Dikarya|Ascomycota|mitosporic Ascomycota|Acremonium'],
            ['Enterobacter agglomerans', 'root|cellular organisms|Bacteria|Proteobacteria|Gammaproteobacteria|Enterobacteriales|Enterobacteriaceae|Pantoea'],
            ['Francisella novicida', 'root|cellular organisms|Bacteria|Proteobacteria|Gammaproteobacteria|Thiotrichales|Francisellaceae|Francisella|Francisella tularensis'],
            ['Glomerella lagenarium', 'root|cellular organisms|Eukaryota|Fungi/Metazoa group|Fungi|Dikarya|Ascomycota|Pezizomycotina|Sordariomycetes|Sordariomycetes incertae sedis|Phyllachorales|Phyllachoraceae|Glomerella'],
            ['Granulobacter bethesdensis', 'root|cellular organisms|Bacteria|Proteobacteria|Alphaproteobacteria|Rhodospirillales|Acetobacteraceae|Granulibacter|Granulibacter bethesdensis'],
            ['Hansenula anomala', 'root|cellular organisms|Eukaryota|Fungi/Metazoa group|Fungi|Dikarya|Ascomycota|Saccharomycotina|Saccharomycetes|Saccharomycetales|Saccharomycetaceae|Pichia'],
            ['Mycobacterium paratuberculosis', 'root|cellular organisms|Bacteria|Actinobacteria|Actinobacteria (class)|Actinobacteridae|Actinomycetales|Corynebacterineae|Mycobacteriaceae|Mycobacterium'],
            ['Nocardia aerocolonigenes', 'root|cellular organisms|Bacteria|Actinobacteria|Actinobacteria (class)|Actinobacteridae|Actinomycetales|Pseudonocardineae|Actinosynnemataceae|Lechevalieria'],
            ['Nocardia lactamdurans', 'root|cellular organisms|Bacteria|Actinobacteria|Actinobacteria (class)|Actinobacteridae|Actinomycetales|Pseudonocardineae|Actinosynnemataceae|Lechevalieria'],
            ['Pasteurella piscicida', 'root|cellular organisms|Bacteria|Proteobacteria|Gammaproteobacteria|Vibrionales|Vibrionaceae|Photobacterium|Photobacterium damselae'],
            ['Pelagibacter ubique', 'root|cellular organisms|Bacteria|Proteobacteria|Alphaproteobacteria|Rickettsiales|SAR11 cluster|Candidatus Pelagibacte'],
            ['Penicillium patulum', 'root|cellular organisms|Eukaryota|Fungi/Metazoa group|Fungi|Dikarya|Ascomycota|Pezizomycotina|Eurotiomycetes|Eurotiomycetidae|Eurotiales|Trichocomacea'],
            ['Polyangium cellulosum', 'root|cellular organisms|Bacteria|Proteobacteria|delta/epsilon subdivisions|Deltaproteobacteria|Myxococcales|Sorangineae|Polyangiaceae|Sorangium|Sorangium cellulosum'],
            [' Polyangium cellulosum', 'root|cellular organisms|Bacteria|Proteobacteria|delta/epsilon subdivisions|Deltaproteobacteria|Myxococcales|Sorangineae|Polyangiaceae|Sorangium|Sorangium cellulosum'],
            ['Protochlamydia amoebophila', 'root|cellular organisms|Bacteria|Chlamydiae/Verrucomicrobia group|Chlamydiae|Chlamydiae (class)|Chlamydiales|Parachlamydiaceae|Candidatus Protochlamydia'],
            ['Pseudomonas indigofera', 'root|cellular organisms|Bacteria|Proteobacteria|Betaproteobacteria|Neisseriales|Neisseriaceae|Vogesella'],
            ['Rhizobium fredii', 'root|cellular organisms|Bacteria|Proteobacteria|Alphaproteobacteria|Rhizobiales|Rhizobiacea'],
            ['Rhizobium loti', 'root|cellular organisms|Bacteria|Proteobacteria|Alphaproteobacteria|Rhizobiales|Rhizobiacea'],
            ['Rhizobium meliloti', 'root|cellular organisms|Bacteria|Proteobacteria|Alphaproteobacteria|Rhizobiales|Rhizobiacea'],
            ['Rhodocyclus gelatinosus', 'root|cellular organisms|Bacteria|Proteobacteria|Betaproteobacteria|Burkholderiales'],
            ['Salmonella paratyphi-a', 'root|cellular organisms|Bacteria|Proteobacteria|Gammaproteobacteria|Enterobacteriales|Enterobacteriaceae|Salmonella'],
            ['Streptomyces capreolus', 'root|cellular organisms|Bacteria|Actinobacteria|Actinobacteria (class)|Actinobacteridae|Actinomycetales|Pseudonocardineae|Actinosynnemataceae|Saccharothrix|Saccharothrix mutabilis'],
            ['Streptomyces sphaeroides', 'root|cellular organisms|Bacteria|Actinobacteria|Actinobacteria (class)|Actinobacteridae|Actinomycetales|Streptomycineae|Streptomycetaceae|Streptomyces'],
            ['Streptomyces violaceoniger', 'root|cellular organisms|Bacteria|Actinobacteria|Actinobacteria (class)|Actinobacteridae|Actinomycetales|Streptomycineae|Streptomycetaceae|Streptomyces'],
            ['Streptoverticillium netropsis', 'root|cellular organisms|Bacteria|Actinobacteria|Actinobacteria (class)|Actinobacteridae|Actinomycetales|Streptomycineae|Streptomycetaceae|Streptomyces'],
            ['Treponema hyodysenteriae', 'root|cellular organisms|Bacteria|Spirochaetes|Spirochaetes (class)|Spirochaetales|Brachyspiraceae|Brachyspira'],
            ['Trichoderma harzianum', 'root|cellular organisms|Eukaryota|Fungi/Metazoa group|Fungi|Dikarya|Ascomycota|Pezizomycotina|Sordariomycetes|Hypocreomycetidae|Hypocreales'],
            ['Vibrio anguillarum', 'root|cellular organisms|Bacteria|Proteobacteria|Gammaproteobacteria|Vibrionales|Vibrionaceae|Listonella'],
            ['Vibrio marinus', 'root|cellular organisms|Bacteria|Proteobacteria|Gammaproteobacteria|Alteromonadales|Moritellaceae|Moritella'],
            ['uncultured Porifera', 'root|cellular organisms|Eukaryota|Fungi/Metazoa group|Metazoa|Porifera|environmental samples|uncultured Porifera'],
            ['uncultured bacterial', 'root|cellular organisms|Bacteria|environmental samples'],
            ['uncultured bacterium', 'root|cellular organisms|Bacteria|environmental samples'],
            ['uncultured crenarchaeote', 'root|cellular organisms|Archaea|Crenarchaeota|environmental samples'],

            ['uncultured proteobacterium', 'root|cellular organisms|Bacteria|Proteobacteria|environmental samples'],
            ['uncultured gamma', ''],
            ['uncultured marine', ''],
            ['uncultured methanogenic', ''],
            ['Ascochyta rabiei', 'root|cellular organisms|Eukaryota|Fungi/Metazoa group|Fungi|Dikarya|Ascomycota|Pezizomycotina|Dothideomycetes|Dothideomycetes incertae sedis|Didymella|Didymella rabiei'],
            ['Beauveria bassiana', 'root|cellular organisms|Eukaryota|Fungi/Metazoa group|Fungi|Dikarya|Ascomycota|Pezizomycotina|Sordariomycetes|Hypocreomycetidae|Hypocreales|Clavicipitaceae|mitosporic Clavicipitaceae|Beauveria'],
            ['Botrytis cinerea', 'root|cellular organisms|Eukaryota|Fungi/Metazoa group|Fungi|Dikarya|Ascomycota|Pezizomycotina|Leotiomycetes|Helotiales|Sclerotiniaceae|mitosporic Sclerotiniaceae|Botrytis|Botrytis fuckeliana'],
            ['Brevibacterium flavum', 'root|cellular organisms|Bacteria|Actinobacteria|Actinobacteria (class)|Actinobacteridae|Actinomycetales|Corynebacterineae|Corynebacteriaceae|Corynebacterium|[Brevibacterium] flavum'],
            ['Canis familiaris', 'root|cellular organisms|Eukaryota|Fungi/Metazoa group|Metazoa|Eumetazoa|Bilateria|Coelomata|Deuterostomia|Chordata|Craniata|Vertebrata|Gnathostomata|Teleostomi|Euteleostomi|Sarcopterygii|Tetrapoda|Amniota|Mammalia|Theria|Eutheria|Laurasiatheria|Carnivora|Caniformia|Canidae|Canis|Canis lupus|Canis lupus familiaris'],
            ['Chlamydia pneumoniae', 'root|cellular organisms|Bacteria|Chlamydiae/Verrucomicrobia group|Chlamydiae|Chlamydiae (class)|Chlamydiales|Chlamydiaceae|Chlamydophila|Chlamydophila pneumoniae'],
            ['Chlamydia trachomatis', 'root|cellular organisms|Bacteria|Chlamydiae/Verrucomicrobia group|Chlamydiae|Chlamydiae (class)|Chlamydiales|Chlamydiaceae|Chlamydia|Chlamydia trachomatis'],
            ['Halobacterium salinarium', 'root|cellular organisms|Archaea|Euryarchaeota|Halobacteria|Halobacteriales|Halobacteriaceae|Halobacterium'],
            ['Monascus anka', 'root|cellular organisms|Eukaryota|Fungi/Metazoa group|Fungi|Dikarya|Ascomycota|Pezizomycotina|Eurotiomycetes|Eurotiomycetidae|Eurotiales|Elaphomycetaceae|Monascus|Monascus purpureus'],
            ['Oscillatoria agardhii', 'root|cellular organisms|Bacteria|Cyanobacteria|Oscillatoriales|Planktothrix|Planktothrix agardhii'],
            ['Psychrobacter arcticum', 'root|cellular organisms|Bacteria|Proteobacteria|Gammaproteobacteria|Pseudomonadales|Moraxellaceae|Psychrobacter'],
            ['Ruegeria gelatinovora', 'root|cellular organisms|Bacteria|Proteobacteria|Alphaproteobacteria|Rhodobacterales|Rhodobacteraceae|Thalassobius|Thalassobius gelatinovorus'],
            ['Streptomycetes hygroscopiocus', 'root|cellular organisms|Bacteria|Actinobacteria|Actinobacteria (class)|Actinobacteridae|Actinomycetales|Streptomycineae|Streptomycetaceae|Streptomyces|Streptomyces hygroscopicus'],
            ['Fragaria ananassa', 'root|cellular organisms|Eukaryota|Viridiplantae|Streptophyta|Streptophytina|Embryophyta|Tracheophyta|Euphyllophyta|Spermatophyta|Magnoliophyta|eudicotyledons|core eudicotyledons|rosids|eurosids I|Rosales|Rosaceae|Rosoideae|Fragaria'],
            ['Ruthia magnifica', ''],
            ['Vesicomyosocius okutanii', '']
            ]

if __name__ == '__main__': main()
