#!/usr/bin/env python
"""
FUNCTION: Adds manual annotations of HMM hits to the database. The
          input is coming from an CSV file.

USAGE: PKS_db_add_tree_based_hit_annotations.py <DB_FILE> <CSV_FILE>  <DB_SWITCH>

       The <CVS_FILE> needs to contain the colloing colums:
         - HMM hit name id
         - Domain name
         - Annotation group
         - Comment (can be empty)

       <DB_SWITCH> can be
         - 'insert' to put new data into the database
         - 'update' to change entries in the database

Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import pks_db_tools

def main():
    if (not len(sys.argv) == 4) or (not sys.argv[3] in ['insert', 'update']):
        print __doc__
        sys.exit(2)
    adder = TreeBasedHitAnnotationAdder(sys.argv[1])
    adder.read_CSV_file(sys.argv[2], sys.argv[3])

class TreeBasedHitAnnotationAdder(object):
    """
    Adds the content of a CSV file to the database. The date is the
    grouping of the HMM hits based on the tree position.
    The CVS file must have the following columns:
    - HMM hit name id
    - Domain name
    - Annotation group
    - Comment (can be empty)

    """

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
            
    def read_CSV_file(self, CVS_file, insert_update_switch):
        self.db_connector.create_db_connection()
        con = self.db_connector.get_connection()
        for line in open(CVS_file):
            line = line.replace('"','')
            line = line.replace("'",'')
            if line[0] in ['#', '\n'] or line.startswith('hit name'): continue
            hit_name, domain_HMM_name, annotation_group, comment = line[
                :-1].split('\t')
            domain_HMM_hit_name  = "%s_%s" % (domain_HMM_name, hit_name)

            if insert_update_switch == 'insert':
                self.db_connector.fill_tree_based_hit_annotations_table(
                    domain_HMM_hit_name, annotation_group, comment)
            elif insert_update_switch == 'update':
                self.db_connector.update_tree_based_hit_annotations_table(
                    domain_HMM_hit_name, annotation_group, comment)

if __name__ == '__main__': main()
