#!/usr/bin/env python
"""
FUNCTION: Adds the relative position of the HMM hits in the tree into
          the database. For this it reads an SVG file of the tree in
          linear form produced by iTOL.

USAGE: PKS_db_add_tree_positions.py <DB_FILE> <LINEAR_TREE_SVG> <DOMAIN_NAME> <DB_SWITCH>

       <DB_SWITCH> can be
         - 'insert' to put new data into the database
         - 'update' to change entries in the database

Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import pks_db_tools

def main():
    if (not len(sys.argv) == 5) or (not sys.argv[4] in ['insert', 'update']):
        print __doc__
        sys.exit(2)
    adder = TreePositionAdder(sys.argv[1])
    adder.read_tree_svg_file(sys.argv[2], sys.argv[3], sys.argv[4])

class TreePositionAdder(object):

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)

    def read_tree_svg_file(self, tree_svg_file, domain_name,
                           insert_update_switch):
        """Read the SVG file of the liste (not circular) tree

        The y-position are used to sort the tree nodes.

        """

        print 'Hi! I hope you have used a LINEAR SVG tree version, slacker!'

        self.__check_domain_name(domain_name)
        number_headers_and_y_coordinates = []
        for line in open(tree_svg_file):
            if line.startswith('<text xml:space="preserve" id="') or \
                   line.startswith('<text xml:space="preserve"  id="'):
                number_headers = line.split('">')[1].split('<')[0]
                y_coordinate = float(line.split('y="')[1].split('"')[0])
                number_headers_and_y_coordinates.append([y_coordinate,
                                                         number_headers])
        number_headers_and_y_coordinates.sort()
        counter = 1
        self.db_connector.create_db_connection()
        con = self.db_connector.get_connection()
        cur = con.cursor()
        for y_coordinate, number_header in number_headers_and_y_coordinates:
            # Get hit_name via number id
            cur.execute('SELECT domain_HMM_hit_name, HMM_hit_name '
                        'FROM HMM_hits WHERE '
                        'simplified_hit_header="%s" AND '
                        'HMM_domain_name="%s";' % (number_header, domain_name))
            try:
                domain_HMM_hit_name, HMM_hit_name = cur.fetchone()
            except:
                print 'Error! No hit with simplified_hit_header = "%s" '\
                      'and domain ="%s" in the database.' % (
                    number_header, domain_name)

            if insert_update_switch == 'insert':
                # Fill into the tree position table
                self.db_connector.fill_tree_position_table(
                    domain_HMM_hit_name, counter)
            elif insert_update_switch == 'update':
                self.db_connector.update_tree_position_table(
                    domain_HMM_hit_name, counter)

            counter += 1
        if counter == 1:
            print 'Warning: No tree positions were found and added!'
            
    def __check_domain_name(self, domain_name):
        valid_domain_names = ['AT', 'DH', 'ER', 'KR', 'KS', 'MT', 'PP', 'TE']
        if domain_name not in valid_domain_names:
            print 'Domain name "%s" is not valid!' % domain_name
            sys.exit(2)

if __name__ == '__main__': main()
