#!/usr/bin/env python
"""
FUNCTION: Compares the domain combination of protein from environemtal
          samples with UniRef proteins. The aim is to find new
          combinations.

USAGE: PKS_db_analyse_domain_combinations.py <DB_FILE> <MIN_GPHS>

Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import re
import sys

import pks_db_tools

def main():
    if not len(sys.argv) == 3:
        print __doc__
        sys.exit(2)
    analyser = DomainComboAnalyser(sys.argv[1])
    analyser.analyse_domain_combinations(sys.argv[2])

class DomainComboAnalyser(object):

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.db_connector.create_db_connection()
        self.replace_pattern = re.compile('\(\s+?\)')

    def analyse_domain_combinations(self, min_GPHS):
        self._get_domain_combos(min_GPHS)
        #self.env_proteins_and_domains['This_is_just_a_test'] = 'AT:XX'
        for env_protein in self.env_proteins_and_domains.keys():
            domain_composition = self.env_proteins_and_domains[env_protein]
            state = 'not_found_in_UniRef'
            # Search the domain combination in all UniRef proteins
            for UniRef_protein in self.UniRef_proteins_and_domains.keys():
                if domain_composition in self.UniRef_proteins_and_domains[
                    UniRef_protein]:
                    state = 'found_in_UniRef'
                    break
            if state == 'not_found_in_UniRef':
                print 'Found new domain combination in protein "%s": %s' % (
                    env_protein, domain_composition)

    def _get_domain_combos(self, min_GPHS):
        self.env_proteins_and_domains = {}
        self.UniRef_proteins_and_domains = {}
        min_GPHS = float(min_GPHS)
        for protein in self.db_connector.get_all_used_proteins():
            source = self.db_connector.get_source_of_protein(protein)
            # Skipt PKSDB proteins
            if source == 'PKSDB': continue
            # Skipt low GPHS proteins
            if self.db_connector.calc_GPHS < min_GPHS: continue
            domain_composition = (
                self.db_connector.build_domain_composition_string(protein))
            domain_composition = self._remove_anno_groups_from_string(
                domain_composition)
            # Skipt single domain hit proteins
            if len(domain_composition) == 2: continue 
            if source == 'UniRef':
                self.UniRef_proteins_and_domains[protein] = domain_composition
            else:
                self.env_proteins_and_domains[protein] = domain_composition

    def _remove_anno_groups_from_string(self, domain_composition):
        domain_composition = re.sub('\(.+?\)', '', domain_composition)
        return(domain_composition)

if __name__ == '__main__': main()
