#!/usr/bin/env python
"""
FUNCTION: Creates for every domain two files - one for hits marked as
          PKS, one for hits marked as Non-PKS - containing the bit
          scores of the hits. These list are then used to create box
          plots with R.
          
USAGE: PKS_db_create_bit_score_countings.py <DB_FILE>
         
Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import pks_db_tools

def main():
    if not len(sys.argv) == 2:
        print __doc__
        sys.exit(2)
    bitscore_list_creator = BitScoreListCreator(sys.argv[1])
    bitscore_list_creator.create_lists()
    bitscore_list_creator.create_R_file()
    
class BitScoreListCreator(object):

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()

    def create_lists(self):
        self.used_domains = []
        for domain in self.db_connector.get_all_used_domains():
            self.used_domains.append(domain)
            pks_fh = open('bitscores_%s_PKS.txt' % domain, 'w')
            non_pks_fh = open('bitscores_%s_Non-PKS.txt' % domain, 'w')
            for hit in self.db_connector.get_all_hits_of_domain(domain):
                bitscore = self.db_connector.get_hit_features(hit)['bitscore']
                annotation = self.db_connector.get_tree_based_annotation_of_hit(
                    hit)
                if annotation == 'PKS':
                    pks_fh.write('%s\n' % bitscore)
                elif annotation == 'Non-PKS':
                    non_pks_fh.write('%s\n' % bitscore)
                else:
                    print 'No tree based annotation for hit "%s"!'
            pks_fh.close()
            non_pks_fh.close()

    def create_R_file(self):
        self.used_domains.sort()
        R_fh = open('bitscores_boxplotter.R', 'w')
        domain_anno_vars = []
        domain_anno_names = []
        for domain in self.used_domains:
            for annotation in ['PKS', 'Non-PKS']:
                R_fh.write('%s_%s=read.table("bitscores_%s_%s.txt")\n' % (
                    domain, annotation.replace('-','_'), domain, annotation))
                domain_anno_vars.append(
                    '%s_%s$V1' % (domain, annotation.replace('-','_')))
                domain_anno_names.append('"%s %s"' % (domain, annotation))
        R_fh.write('pdf("bitscores_boxplots.pdf", width=18)\n')
        R_fh.write('boxplot(%s, main="%s", names = c(%s), ylab = "Bitscore")\n' % (
            ', '.join(domain_anno_vars),
            'Bit score distibutions',
            ', '.join(domain_anno_names)))
        R_fh.close()

if __name__ == '__main__': main()
