#!/usr/bin/env python
"""
FUNCTION: Creates a very detailed report of PKS/Non-PKS hit in different
          source and domains.

USAGE: PKS_db_create_detailed_hit_statistics.py <DB_FILE>

Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import pks_db_tools


def main():
    if not len(sys.argv) == 2:
        print __doc__
        sys.exit(2)
    hit_stat_creator = HitStatCreator(sys.argv[1])
    hit_stat_creator.perform_countings()
    hit_stat_creator.create_output_CVS_file()
    
class HitStatCreator(object):
    """

    """
    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        self.cur = self.con.cursor()

    def perform_countings(self):
        self.domains_sources_hits = {}
        self.used_sources = []
        for domain in self.db_connector.get_all_used_domains():
            self.domains_sources_hits[domain] = {}
            for hit in self.db_connector.get_all_hits_of_domain(domain):
                source = self.db_connector.get_source_of_hit(hit)
                tree_anno = (
                    self.db_connector.get_tree_based_annotation_of_hit(hit))
                if source in [None]: continue
                if source not in self.used_sources:
                    self.used_sources.append(source)
                if not self.domains_sources_hits[domain].has_key(source):
                    self.domains_sources_hits[domain][source] = {}
                if not self.domains_sources_hits[domain][source].has_key(
                    tree_anno):
                    self.domains_sources_hits[domain][source][tree_anno] = 1
                else:
                    self.domains_sources_hits[domain][source][tree_anno] += 1

    def create_output_CVS_file(self):
        output_file_name = 'PKS_hit_detailed_counting_results.csv'
        self.output_file = open(output_file_name, 'w')
        self.used_sources.sort()

        # Hard coded domain, source and tree_anno names! Adapt if data changes.
        used_sources = ['PKSDB', 'UniRef', 'MFS', 'SGS', 'EBPRS', 'WLF', 'AMD', 'HGUT'] 
        used_domains = ['AT', 'DH', 'ER', 'KR', 'KS', 'MT', 'PP', 'TE']
        used_tree_annos = ['PKS', 'Non-PKS']
        
        self.domain_sums = {}
        # Header line
        self.output_file.write(';')
        for domain in used_domains:
            for tree_anno in used_tree_annos:
                self.output_file.write('%s-%s;' % (domain, tree_anno))
        self.output_file.write(
            'summed_PKS_of_source;summed_Non-PKS_of_source;\n')

        # Result lines
        for source in used_sources:
            self.sum_of_PKS_for_source = 0
            self.sum_of_Non_PKS_for_source = 0
            self.output_file.write('%s;' % source)
            for domain in used_domains:
                for tree_anno in used_tree_annos:
                    self._print_counting_cell(source, domain, tree_anno)
            # Print the two cells with the sums at the end of the line
            self.output_file.write('%s;%s\n' % (
                self.sum_of_PKS_for_source, self.sum_of_Non_PKS_for_source))

        # Line of the domain sums (column)
        self.output_file.write('Sum_over_all_sources;')
        for domain in used_domains:
            for tree_anno in used_tree_annos:
                self.output_file.write('%s;' % (self.domain_sums['%s-%s' % (domain, tree_anno)]))
        self.output_file.write('\n')

    def _print_counting_cell(self, source, domain, tree_anno):
        counting = 0
        # The cell printin itself
        if not self.domains_sources_hits[domain].has_key(source):
            pass
        elif self.domains_sources_hits[domain][source].has_key(tree_anno):
            counting = self.domains_sources_hits[domain][source][tree_anno]
            if tree_anno == 'PKS': self.sum_of_PKS_for_source += counting
            elif tree_anno == 'Non-PKS': self.sum_of_Non_PKS_for_source += counting
        self.output_file.write('%s;' % counting)
            
        # Add to domain (column) sum - for the last line
        if self.domain_sums.has_key('%s-%s' % (domain, tree_anno)):
            self.domain_sums['%s-%s' % (domain, tree_anno)] += counting
        else:
            self.domain_sums['%s-%s' % (domain, tree_anno)] = counting

if __name__ == '__main__': main()
