#!/usr/bin/env python
"""
FUNCTION: Counts the number of PKSs and other hits.

USAGE: PKS_db_create_hit_statistics.py <DB_FILE>

Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import pks_db_tools


def main():
    if not len(sys.argv) == 2:
        print __doc__
        sys.exit(2)
    hit_stat_creator = HitStatCreator(sys.argv[1])
    hit_stat_creator.perform_countings()
    hit_stat_creator.create_output_CVS_file()
    
class HitStatCreator(object):
    """

    """

    proteins_with_hits = {}

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        self.cur = self.con.cursor()

    def perform_countings(self):
        # A dictionary of dictionaries of dictionaries
        # domain -> source -> tree based annotation -> counting
        self.domains_sources_hits = {}
        self.used_sources = []
        for domain in self.db_connector.get_all_used_domains():
            self.domains_sources_hits[domain] = {}
            for hit in self.db_connector.get_all_hits_of_domain(domain):
                source = self.db_connector.get_source_of_hit(hit)
                tree_anno = (
                    self.db_connector.get_tree_based_annotation_of_hit(hit))
                # Skip PKSDB and String
                if source in [None, 'PKSDB', 'String']: continue
                if source not in self.used_sources:
                    self.used_sources.append(source)
                if not self.domains_sources_hits[domain].has_key(source):
                    self.domains_sources_hits[domain][source] = {}
                if not self.domains_sources_hits[domain][source].has_key(
                    tree_anno):
                    self.domains_sources_hits[domain][source][tree_anno] = 1
                else:
                    self.domains_sources_hits[domain][source][tree_anno] += 1

    def create_output_CVS_file(self):
        output_file_name = 'PKS_hit_counting_results.csv'
        output_file = open(output_file_name, 'w')
        self.used_sources.sort()
        # Write header line
        for source in self.used_sources:
             output_file.write(';%s' % source)
        output_file.write('\n')
        # Write line for every domain
        for domain in sorted(self.domains_sources_hits.keys()):
            output_file.write(domain)
            for source in self.used_sources:
                
                if not self.domains_sources_hits[domain].has_key(source) or \
                       not self.domains_sources_hits[domain][source].has_key(
                    'PKS'):
                    output_file.write(';0')
                else:
                    output_file.write(
                        ';%s' % self.domains_sources_hits[domain][source]['PKS'])
            output_file.write('\n')
        output_file.close()
        print 'Wrote output to file "%s".' % output_file_name

if __name__ == '__main__': main()
