#!/usr/bin/env python

"""
FUNCTION: Uses the datbase to create files describen the color code of
          tree nodes in iTOL.

USAGE: PKS_db_create_overview_CSV_file.py <DB_FILE> <DOMAIN>
         
Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import pks_db_tools

def main():
    if not len(sys.argv) == 3:
        print __doc__
        sys.exit(2)

    creator = ITOLColorFileCreator(sys.argv[1], sys.argv[2])
    creator.get_domain_HMM_hit_names_and_simp_headers()
    creator.create_taxonomy_color_file()
    creator.create_protein_function_color_file()
    creator.create_environments_color_file()
    creator.create_full_protein_tree_based_annotation_color_file()
    creator.create_bitscore_color_file()

class ITOLColorFileCreator(object):

    output_file_prefix = 'PKS_iTOL_color'
    annotation_group_to_color_code = {
        'PKSDB' : '#ff0000', # red
        'PKS' : '#ff8800', # orange
        'Putative_PKS' : '#ffff00', # yellow
        'Putative_FAS' : '#00ff00', # light green
        'FAS' : '#00AA00', # green 
        'COG_FAS' : '#008800', # green
        'Acyl_synthase' : '#CFCFCF', # light grey
        'Lipopeptide' : '#CFCFCF', # light grey
        'NRPS' : '#CFCFCF',# light grey
        'Oxidoreductase_dehydrogenase' : '#CFCFCF', # light grey
        'Thioesterase' : '#CFCFCF', # light grey
        'Hydrolase_esterase' : '#CFCFCF', # light grey
        'Amino_acid_adenylation' : '#CFCFCF', # light grey
        'Unknown' : '#000000',  # black
        'Hypothetical_protein' : '#000000', # black
        }

    environment_to_color_code = {
        'SGS' : '#0066FF', # blue
        'MFS' : '#FFC105', # orange
        'WLF' : '#99EBFF', # turcis
        'EBPRS' : '#8AB800', # green
        'HGUT' : '#996600', # brown
        'AMD' : '#FF6694', # viollet
        }

    phyla_to_color = {
        'Actinobacteria' : '#FF9999', # red
        'Proteobacteria' : '#FFE5FF', # light-violett
        'Fungi' : '#C2FFC2', # green
        'Cyanobacteria' : '#CC99FF', 
        'Firmicutes' : '#AB5959',# brown-red
        'Mycetozoa' : '#FFD24D', # light orange
        'Alveolata' : '#6BDAFF', # blue-turkis
        'Viridiplantae' : '#ADD4AA', #darker-green
        'Animal' : '#FF3399',  # olive
        'Euglenozoa' : '#BBB777', # beige
        'Chloroflexi' : '#77BBBB', # grey-turkis
        'Planctomycetes' : '#D4AAAA', 
        'Others_eukaryotes' : '#CCCCCC', # maybe improve
        'Others' : '#CCCCCC', # maybe improve

        
        # NOT USED:
        'Unclassified eukaryotes' : '#FFCCF5',  # viollet
        'Aquificae' : '#AAAAAA',
        'Bacillus/Clostridium' : '#AAAAAA',
        'Bacteroidetes/Chlorobi' : '#AAAAAA',
        'Deinococcus-Thermus': '#AAAAAA',
        'Fibrobacter/Acidobacteria': '#AAAAAA',
        'Fusobacteria' : '#AAAAAA',
        'Spirochaetes' : '#AAAAAA',
        'Archaea' : '#CCCCCC', # 

        'Chromophyta' : '#CCCCCC', # maybe improve
        'Chlamydiae/Verrucomicrobia' : '#CCCCCC', # maybe improve
        #'Thermotogae' : '#CCCCCC', # maybe improve
        'Unclassified Prokaryotae' : '#CCCCCC', # maybe improve
        'Others_bacteria' : '#CCCCCC', # maybe improve
        'Viruses' : '#CCCCCC', # IMPROVE
        }

    def __init__(self, db_file, domain):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.domain = domain

    def create_protein_function_color_file(self):
        """Creates an iTOL color file to colorize the node ring based
        on the functional groups that determined by anaylizing the
        protein annotation string. Only done for UniRef and STRING
        proteins.

        """
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        cur = self.con.cursor()
        output_file = open('%s_%s.protein_function' % (
            self.output_file_prefix, self.domain), 'w')
        
        for domain_HMM_hit_name, hit_protein_name, simplified_headers in (
            self._get_data_triples(cur)):
            annotation = self.db_connector.get_annotation_string_of_protein(
                hit_protein_name)
            if annotation == None: annotation_group = '-'
            source = self.db_connector.get_source_of_protein(hit_protein_name)
            # Skip environments here
            if source not in ['UniRef', 'String', 'PKSDB']:
                continue
            # PKSDB members 
            elif source == 'PKSDB':
                annotation_group = 'PKSDB'
            # UniRef and String
            else:
                annotation_group = (
                    self.db_connector.get_text_string_annotation_group_of_protein(
                    hit_protein_name))
                
            if not annotation_group == '-':
                color = self._get_function_color(annotation_group)

                output_file.write('%s\t%s\t%s\n' % (
                    simplified_headers,
                    color,
                    annotation_group))
                
        output_file.close()
        self.db_connector.close_db_connection()

    def create_taxonomy_color_file(self):
        """Creates an iTOL color file to colorize the node ring based
        on the taxonomic group of the species.
        """
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        cur = self.con.cursor()
        output_file = open('%s_%s.taxonomic_groups' % (
            self.output_file_prefix, self.domain), 'w')
        for domain_HMM_hit_name, hit_protein_name, simplified_headers in (
            self._get_data_triples(cur)):
            species = self.db_connector.get_species_of_protein(hit_protein_name)
            tax_string = self.db_connector.get_full_tax_of_species(species)
            tax_group = '-'
            color = '#0000000'
            if not tax_string == None:
                tax_group = self._get_phylo_group_from_tax_string(tax_string)
            if not tax_group == '-':
                color = self._get_taxonomy_color(tax_group)
                output_file.write ("%s\t%s\t%s (%s)\n" % (
                    simplified_headers, color, tax_group, species))
        output_file.close()          
        self.db_connector.close_db_connection()

    def create_environments_color_file(self):
        """Creates an iTOL color file to colorize the node ring based
        on the environment a protein hit is coming from.

        """
        
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        cur = self.con.cursor()
        output_file = open('%s_%s.environment' % (
            self.output_file_prefix, self.domain), 'w')

        for domain_HMM_hit_name, hit_protein_name, simplified_headers in (
            self._get_data_triples(cur)):
            source = self.db_connector.get_source_of_protein(hit_protein_name)
            if source == None: continue
            if self.environment_to_color_code.has_key(source):
                output_file.write ("%s\t%s\n" % (
                    simplified_headers,
                    self.environment_to_color_code[source]))
        output_file.close()
        self.db_connector.close_db_connection()

    def create_full_protein_tree_based_annotation_color_file(self):
        """Creates an iTOL color file to colorize the node ring based
        on the tree based annotation of other hits in the same protein
        the hit was found in.
        """

        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        cur = self.con.cursor()
        output_file = open('%s_%s.tree_based_full_protein_annotation' % (
            self.output_file_prefix, self.domain), 'w')

        for domain_HMM_hit_name, hit_protein_name, simplified_headers in \
            self._get_data_triples(cur):
            tree_based_annotation_countings = {
                'PKS' : 0, 'others' : 0, 'unknown': 0}
            # Get all hits of the protein
            hits = self._get_all_hits_of_protein(hit_protein_name, cur)
            # There must be at least one other hit in the protein
            if len(hits) == 1: continue
            # Get the tree based annotation of the protein
            for hit in hits:
                tree_based_annotation = self._get_tree_based_annotation(
                    hit, cur)
                tree_based_annotation_countings[tree_based_annotation] += 1
            # Calculate score
            protein_anno_score = self._calc_protein_annotation_score(
                tree_based_annotation_countings)
            # Create color code
            color_code = self._protein_annotation_score_to_color_code(
                protein_anno_score)
            output_file.write ("%s\t%s\tPKS: %s / others: %s\n" % (
                    simplified_headers,
                    color_code,
                    tree_based_annotation_countings['PKS'],
                    tree_based_annotation_countings['others']))

    def create_bitscore_color_file(self):
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        cur = self.con.cursor()
        output_file = open('%s_%s.bitscores' % (
            self.output_file_prefix, self.domain), 'w')
        min_bitscore = 0
        max_bitscore = 0
        simplified_headers_and_bitscores = []
        # Get ids, bitscore and find the max and min bitscore
        for domain_HMM_hit_name, hit_protein_name, simplified_headers in \
            self._get_data_triples(cur):
            hit_features = self.db_connector.get_hit_features(
                domain_HMM_hit_name)
            bitscore = float(hit_features['bitscore'])
            simplified_headers_and_bitscores.append(
                [simplified_headers, bitscore])
            if bitscore < min_bitscore:
                min_bitscore = bitscore
            if bitscore > max_bitscore:
                max_bitscore = bitscore
        # Calculate the normalized bitscore * color value
        for simplified_headers, bitscore in simplified_headers_and_bitscores:
            normalized_bitscore = 0
            if min_bitscore >= 0:
                try:
                    normalized_bitscore = (
                        (bitscore - min_bitscore) / (max_bitscore - min_bitscore))
                except:
                    normalized_bitscore = 0
            elif min_bitscore < 0:
                try:
                    normalized_bitscore = (
                        (bitscore + abs(min_bitscore)) /
                        (max_bitscore + abs(min_bitscore)))
                except:
                    normalized_bitscore = 0
            color_code = None
            if normalized_bitscore != 0:
                color_code = int(255 - normalized_bitscore * 255)
            else:
                color_code = int(255)
            hex_color_code = '#%02x%02x%02x' % (
                color_code, color_code, color_code)
            output_file.write (
                "%s\t%s\tbitscore: %s, normalized bitscrore: %s\n" % (
                simplified_headers, hex_color_code, bitscore, normalized_bitscore))
                    
    def _protein_annotation_score_to_color_code(self, score):
        color_code = '#CCCCCC'
        if score == None:
            return(color_code)
        if score > 0:
            red_value_dec = int(score * 255)
            # If the value goes too low the color would become to dark
            if red_value_dec < 120: red_value_dec = 100
            # Transform to hex
            red_value_hex = str(hex(red_value_dec))[-2:]
            color_code = '#%s0000' % red_value_hex
        elif score < 0:
            green_value_dec = int(-score * 255)
            # If the value goes too low the color would become to dark
            if green_value_dec < 120: red_value_dec = 100
            # Transform to hex
            green_value_hex = str(hex(green_value_dec))[-2:]
            color_code = '#00%s00' % green_value_hex
        return(color_code)

    def _calc_protein_annotation_score(self, countings):
        PKS_annos = float(countings['PKS'])
        other_annos = float(countings['others'])
        score = ''
        try: 
            score = ((PKS_annos - other_annos) /
                     (PKS_annos + other_annos))
        except ZeroDivisionError:
            score = None
        return(score)

    def _get_tree_based_annotation(self, domain_HMM_hit_name, cur):
        cur.execute('SELECT tree_based_annotation_group FROM '
                    'tree_based_hit_annotations WHERE '
                    'domain_HMM_hit_name="%s";' % (
            domain_HMM_hit_name))
        potential_tree_based_annotation = cur.fetchone()
        if potential_tree_based_annotation == None:
            return('unknown')
        elif potential_tree_based_annotation[0] == 'PKS':
            return('PKS')
        else:
            return('others')

    def _get_all_hits_of_protein(self, hit_protein_name, cur):
        hits = []
        cur.execute('SELECT domain_HMM_hit_name FROM HMM_hits '
                    'WHERE hit_protein_name="%s";' % (
            hit_protein_name))
        for hit in cur.fetchall():
            hits.append(hit[0])
        return(hits)

    def get_domain_HMM_hit_names_and_simp_headers(self):
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        self.domain_HMM_hit_names_and_simp_headers = {}
        cur = self.con.cursor()
        cur.execute('SELECT domain_HMM_hit_name, simplified_hit_header '
                    'FROM HMM_hits WHERE HMM_domain_name="%s";' % (
            self.domain))
        for domain_HMM_hit_name, simplified_header in cur.fetchall():
            # Discard entries that are added as cluster members; they
            # are not found in the tree.
            if simplified_header == '-': continue
            self.domain_HMM_hit_names_and_simp_headers[
                domain_HMM_hit_name] = simplified_header
        self.db_connector.close_db_connection()

    def _get_data_triples(self, cur):
        for domain_HMM_hit_name in \
                self.domain_HMM_hit_names_and_simp_headers.keys():
            # Get protein name of the hits
            cur.execute('SELECT hit_protein_name FROM HMM_hits '
                        'WHERE domain_HMM_hit_name="%s";' % (
                domain_HMM_hit_name))
            hit_protein_name = cur.fetchone()[0]
            yield(domain_HMM_hit_name, hit_protein_name,
                  self.domain_HMM_hit_names_and_simp_headers[
                domain_HMM_hit_name])

    def _get_function_color(self, annotation_group):
        return(self.annotation_group_to_color_code[annotation_group])

    def _get_taxonomy_color(self, tax_group):
        return(self.phyla_to_color[tax_group])

    def _get_phylo_group_from_tax_string(self, tax_string):
        level_name_group_triples  = [
            [3, 'Proteobacteria', 'Proteobacteria'],
            [3, 'Cyanobacteria', 'Cyanobacteria'],
            [3, 'Bacteroidetes/Chlorobi group', 'Bacteroidetes/Chlorobi'],
            [3, 'Alveolata', 'Alveolata'],
            [3, 'Bacillus/Clostridium group', 'Bacillus/Clostridium'],
            [3, 'Actinobacteria', 'Actinobacteria'],
            [3, 'Deinococcus-Thermus', 'Deinococcus-Thermus'],
            [3, 'Planctomycetes', 'Planctomycetes'],
            [3, 'Aquificae', 'Aquificae'],
            [3, 'Chromophyta', 'Chromophyta'],
            [3, 'Chloroflexi', 'Chloroflexi'],
            [3, 'Mycetozoa', 'Mycetozoa'],
            [3, 'Spirochaetes', 'Spirochaetes'],
            [3, 'Fibrobacter/Acidobacteria group', 'Fibrobacter/Acidobacteria'],
            [3, 'Fusobacteria', 'Fusobacteria'],
            [3, 'Chlamydiae/Verrucomicrobia group',
             'Chlamydiae/Verrucomicrobia'],
            [3, 'Firmicutes', 'Firmicutes'],
            [3, 'Chlorobionta', 'Viridiplantae'],
            [3, 'Viridiplantae', 'Viridiplantae'],
            [1, 'Viruses', 'Viruses'],
            [1, 'other sequences', 'Others'],
            [4, 'Fungi', 'Fungi'],
            [3, 'Not yet classified (Prokaryotae)', 'Unclassified Prokaryotae'],
            [4, 'Animalia', 'Animal'],
            [4, 'Metazoa', 'Animal'],
            [2, 'Archaea', 'Archaea'],
            [3, 'Euglenozoa', 'Euglenozoa'],
            [3, 'Mycetozoa', 'Mycetozoa'],
            [3, 'unclassified eukaryotes', 'Others_eukaryotes'],
            [3, 'unclassified eukaryotes', 'Others_eukaryotes'],
            [3, 'stramenopiles', 'Others_eukaryotes'],
            [3, 'Cercozoa', 'Others_eukaryotes'],
            [3, 'Haptophyceae', 'Others_eukaryotes'],
            [3, 'Cryptophyta', 'Others_eukaryotes'],
            [2, 'Bacteria', 'Others_bacteria'],
            [1, 'unclassified sequences', 'Others'],
            ]
        split_tax_string = tax_string.split('|')
        taxonomy_group = None

        for taxonomy_level, name, resulting_group in level_name_group_triples:
            if taxonomy_level > len(split_tax_string) - 1 : continue
            if split_tax_string[taxonomy_level] == name:
                taxonomy_group = resulting_group
                break
            
        if not taxonomy_group == None:
            return(taxonomy_group)
        else:
            if not len(tax_string) == 0:
                print 'No fitting taxonomy group found for "%s". '\
                      'Please adapt code.' % (tax_string)
            return('-')

if __name__ == '__main__': main()
