#!/usr/bin/env python
"""
FUNCTION: Counts the number of different taxonomic groups

USAGE: PKS_db_create_species_statistics.py <DB_FILE>

Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import pks_db_tools
import PKS_db_create_iTOL_color_files


def main():
    if not len(sys.argv) == 2:
        print __doc__
        sys.exit(2)
    species_stat_creator = SpeciesStatCreator(sys.argv[1])
    species_stat_creator.count_species_hits()
    #species_stat_creator.create_species_hit_statistic()
    #species_stat_creator.create_genus_hit_statistic()
    species_stat_creator.create_higher_tax_level_statistic(sys.argv[1])
    
class SpeciesStatCreator(object):

    proteins_with_hits = {}

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        self.cur = self.con.cursor()

    def count_species_hits(self):
        self.species_and_counts = {}
        self.genus_and_counts = {}
        for hit in self.db_connector.get_all_hits_with_the_given_tree_anno(
            'PKS'):
            protein = self.db_connector.get_protein_of_a_hit(hit)
            species =  self.db_connector.get_species_of_protein(protein)
            genus = None
            if not species == None:
                genus = species.split()[0]
            if self.species_and_counts.has_key(species):
                self.species_and_counts[species] += 1
            else:
                self.species_and_counts[species] = 1
            if self.genus_and_counts.has_key(genus):
                self.genus_and_counts[genus] += 1
            else:
                self.genus_and_counts[genus] = 1

    def create_species_hit_statistic(self):
        counts_and_species = []
        for species in self.species_and_counts.keys():
            counts_and_species.append(
                [self.species_and_counts[species], species])
        counts_and_species.sort()
        counts_and_species.reverse()
        print "+++ Species counting +++"
        print "========================"        
        for counting, species in counts_and_species:
            print species, counting

    def create_genus_hit_statistic(self):
        counts_and_genus = []
        for genus in self.genus_and_counts.keys():
            counts_and_genus.append(
                [self.genus_and_counts[genus], genus])
        counts_and_genus.sort()
        counts_and_genus.reverse()
        print "+++ Genus counting +++"
        print "======================"                
        for counting, genus in counts_and_genus:
            print genus, counting

    def create_higher_tax_level_statistic(self, db_file):
        tax_level_counter = {}
        tax_group_allocator = (
            PKS_db_create_iTOL_color_files.ITOLColorFileCreator(db_file, None))
        for species in self.species_and_counts.keys():
            full_tax_string = self.db_connector.get_full_tax_of_species(species)
            if full_tax_string == None: continue
            tax_level = tax_group_allocator._get_phylo_group_from_tax_string(
                full_tax_string)
            if tax_level_counter.has_key(tax_level):
                tax_level_counter[tax_level] += self.species_and_counts[species]
            else:
                tax_level_counter[tax_level] = self.species_and_counts[species]
        counts_and_tax_level = []
        for tax_level in tax_level_counter.keys():
            counts_and_tax_level.append([tax_level_counter[tax_level], tax_level])
        counts_and_tax_level.sort()
        counts_and_tax_level.reverse()
        for counting, tax_level in counts_and_tax_level:
            print tax_level, counting

if __name__ == '__main__': main()
