#!/usr/bin/env python
"""
FUNCTION: Searches for PKSDB sequences that are tagged as Non-PKS based
          on the tree position.

USAGE: PKS_db_find_PKSDB_seq_marked_as_Non-PKS.py <DB_FILE>

Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import pks_db_tools

def main():
    if not len(sys.argv) == 2:
        print __doc__
        sys.exit(2)
    finder = BadPKSDBSeqFinder(sys.argv[1])
    finder.hunt_badly_marked_PKSDB_seqs()
    
class BadPKSDBSeqFinder(object):

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.db_connector.create_db_connection()

    def hunt_badly_marked_PKSDB_seqs(self):
        print 'The following PKSDB sequences are tagged as Non-PKS:'
        for hit in self.db_connector.get_all_used_hits():
            source = self.db_connector.get_source_of_hit(hit)
            if source == 'PKSDB':
                tree_based_tag = (
                    self.db_connector.get_tree_based_annotation_of_hit(hit))
                if tree_based_tag == 'Non-PKS':
                    print hit

if __name__ == '__main__': main()

