#!/usr/bin/env python

"""
FUNCTION: Finds species that lack a tax string entry.

USAGE: PKS_db_find_species_without_tax_string.py <DB_FILE>

Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import pks_db_tools

def main():
    if not len(sys.argv) == 2:
        print __doc__
        sys.exit(2)

    hunter = SpeciesHunter(sys.argv[1])
    hunter.hunter_species_withouth_tax_strings()

class SpeciesHunter(object):

    species_without_tax_string = []

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        self.cur = self.con.cursor()

    def hunter_species_withouth_tax_strings(self):
        self.cur.execute(
            'SELECT DISTINCT species FROM proteins;')
        for species in self.cur.fetchall():
            species = species[0]
            self._check_tax_string(species)

    def _check_tax_string(self, species):
        self.cur.execute(
            'SELECT full_taxonomie_string FROM species WHERE '
            'species_name="%s";' % (species))
        if self.cur.fetchone() == None:
            self.species_without_tax_string.append(species)
            print species
            
if __name__ == '__main__': main()
