"""
Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2007 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import os
import sys
import time

class FileProcessor:
    """Opens input or output file."""
    open_file = 0

    def __check_type(self, file_name):

        """Check if the filen_name is  a string."""

        if not type(file_name)  == str:
            sys.stderr.write('Argument of FileProcessor must '
                             'be an file name string.\n')
            sys.exit(2)


    def __check_r_w_switch(self, r_w_switch):
        if not r_w_switch in ['r', 'w']:
            sys.stderr.write('Use "r" or "w" to specify the type of file.\n')
            sys.exit(2)

    def __open_input_file(self, file_name):
        try:
            self.open_file = open(file_name, 'r')
        except:
            sys.stderr.write('Error: Unable to open input file "' \
                             + file_name \
                             + '"!\n')
            sys.exit(2)

    def __check_file_existence(self, file_name):
        if os.path.exists(file_name) == False:
            sys.stderr.write('File "%s" does not exists!\n' %
                             file_name)
            sys.exit(2)

    def __open_output_file(self, file_name):
        try:
            self.open_file = open(file_name, 'w')
        except:
            sys.stderr.write('Error: Unable to open output file "' \
                             + file_name \
                             + '"!\n')
            sys.exit(2)

    def __write_metastring_to_output_file(self):

        """Write call info to the output file header."""

        self.open_file.write(
            '# The output was produced by %s\n' \
            '# Complete call: "%s"\n' \
            '# %s\n' % (
            sys.argv[0],
            ' '.join(sys.argv[:]),
            time.strftime('%H:%M:%S %d.%m.%Y %Z')))

    def get_file(self, file_name, r_w_switch):

        self.__check_type(file_name)
        self.__check_r_w_switch(r_w_switch)
        
        if r_w_switch == 'r':
            self.__check_file_existence(file_name)
            self.__open_input_file(file_name)
            
        elif r_w_switch == 'w':
            self.__open_output_file(file_name)
            self.__write_metastring_to_output_file()
        return(self.open_file)
    

    def close_file(self):
        open_file.close()




