#!/usr/bin/env python
"""
FUNCTION: Compares GPHS (global protein hit score) and annotation of
          UniRef proteins.

USAGE: PKS_db_create_protein_hit_GPHS_vs_annotation_overview.py <DB_FILE> <CUT_OFF_GPHS>
         
Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import pks_db_tools


def main():
    if not len(sys.argv) == 3:
        print __doc__
        sys.exit(2)
    protein_stat_creator = HitProteinStatCreator(sys.argv[1])
    protein_stat_creator.perform_countings(sys.argv[2])
    protein_stat_creator.create_output_file()
    
class HitProteinStatCreator(object):

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        self.cur = self.con.cursor()

    def perform_countings(self, cut_off_GPHS_values):
        self.multi_hit_protein_anno_counting = {}
        self.single_hit_protein_anno_counting = {}
        for protein in self.db_connector.get_all_used_proteins():
            source = self.db_connector.get_source_of_protein(protein)
            if not source == 'UniRef': continue
            GPHS = self.db_connector.calc_GPHS(protein)
            annotation_group = (
                self.db_connector.get_text_string_annotation_group_of_protein(
                protein))

            # For protein with more than one domain hit the GPHS
            # is not None and can be used:
            if GPHS >= float(cut_off_GPHS_values):
                if self.multi_hit_protein_anno_counting.has_key(annotation_group):
                    self.multi_hit_protein_anno_counting[annotation_group] += 1
                    print protein
                else:
                    self.multi_hit_protein_anno_counting[annotation_group] = 1
            elif GPHS < float(cut_off_GPHS_values) and not GPHS == None:
                pass
            # For the single domain hit protein no the GPHS is None;
            # then the single hit is used to discriminate
            elif GPHS == None:
                hits = self.db_connector.get_all_hits_of_protein(protein)
                # Just a as check
                if len(hits) > 1:
                    print 'Something is fishy here. This protein should' \
                          'have a GPHS!'
                    continue
                tree_based_annnotation = (
                    self.db_connector.get_tree_based_annotation_of_hit(hits[0]))
                if tree_based_annnotation == 'PKS':
                    if self.single_hit_protein_anno_counting.has_key(
                        annotation_group):
                        self.single_hit_protein_anno_counting[
                            annotation_group] += 1
                    else:
                        self.single_hit_protein_anno_counting[
                            annotation_group] = 1
            
    def create_output_file(self):
        print self.multi_hit_protein_anno_counting
        print self.single_hit_protein_anno_counting


if __name__ == '__main__': main()
