#!/usr/bin/env python
"""
FUNCTION: Counts the number of PKS hits protein (not hit) wise.

USAGE: PKS_db_create_protein_hit_statistics.py <DB_FILE> <CUT_OFF_GPHS>
         
Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import pks_db_tools


def main():
    if not len(sys.argv) == 3:
        print __doc__
        sys.exit(2)
    protein_stat_creator = HitProteinStatCreator(sys.argv[1])
    protein_stat_creator.perform_countings(sys.argv[2])
    protein_stat_creator.create_output_file()
    
class HitProteinStatCreator(object):

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        self.cur = self.con.cursor()

    def perform_countings(self, cut_off_GPHS_values):

        # A dictionary of a dictionaries
        # souces -> tree based annotation -> countings
        self.sources_and_countings = {}
        for protein in self.db_connector.get_all_used_proteins():
            source = self.db_connector.get_source_of_protein(protein)
            GPHS = self.db_connector.calc_GPHS(protein)
            if not self.sources_and_countings.has_key(source):
                self.sources_and_countings[source] = {
                    'PKS': 0, 'PKS_single_hits' : 0, 'non_PKS' : 0}
            annotation_group = ''
            # For protein with more than one domain hit the GPHS
            # is not None and can be used:
            if GPHS >= float(cut_off_GPHS_values):
                annotation_group = 'PKS'
            elif GPHS < float(cut_off_GPHS_values) and not GPHS == None:
                annotation_group = 'non_PKS'
            # For the single domain hit protein no the GPHS is None;
            # then the single hit is used to discriminate
            elif GPHS == None:
                hits = self.db_connector.get_all_hits_of_protein(protein)
                # Just a as control
                if len(hits) > 1:
                    print 'Something is fishy here. This protein should' \
                          'have a GPHS!'
                    continue
                annnotation = self.db_connector.get_tree_based_annotation_of_hit(
                    hits[0])
                if annnotation == 'PKS':
                    annotation_group = 'PKS_single_hits'
                else:
                    annotation_group = 'non_PKS'
            self.sources_and_countings[source][annotation_group] += 1

    def create_output_file(self):
        output_file_name = 'PKS_protein_counting_results.csv'
        output_file = open(output_file_name, 'w')
        # Verbose version
        for source in self.sources_and_countings.keys():
            output_file.write('%s\n' % source)
            for annotation_group in ['PKS', 'PKS_single_hits', 'non_PKS']:
                output_file.write(' - %s: %s\n' % (
                    annotation_group,
                    self.sources_and_countings[source][annotation_group]))
        # Compact version
        for source in self.sources_and_countings.keys():
            output_file.write('%s' % source)
            for annotation_group in ['PKS', 'PKS_single_hits', 'non_PKS']:
                output_file.write(';%s' % (
                    self.sources_and_countings[source][annotation_group]))
            output_file.write('\n')
        print 'Wrote output to file "%s".' % output_file_name

if __name__ == '__main__': main()
