#!/usr/bin/env python
"""
FUNCTION: Creates a CSV dump of selected database content.

USAGE: PKS_db_create_protein_overview_CSV_file.py <DB_FILE> 
         
Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import pks_db_tools


def main():
    if not len(sys.argv) == 2:
        print __doc__
        sys.exit(2)
    protein_overview_creator = ProteinOverviewCreator(sys.argv[1])
    protein_overview_creator.select_PKS_protein()
    protein_overview_creator.create_output_file()
    
class ProteinOverviewCreator(object):

    cut_off_GPHS_value = 0.0

    def __init__(self, db_file):
        self.db_connector = pks_db_tools.PKSDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        self.cur = self.con.cursor()

    def select_PKS_protein(self):
        self.mult_hit_proteins_sorted_by_GPHS = []
        self.single_hit_PKS_proteins = []
        for protein in self.db_connector.get_all_used_proteins():
            # DEVELOPMENT ONLY
            #if not protein.startswith('UniRef100_P'): continue
            GPHS = self.db_connector.calc_GPHS(protein)
            # For protein with more than one domain hit the GPHS
            # is not None and can be used:
            if GPHS >= self.cut_off_GPHS_value:
                self.mult_hit_proteins_sorted_by_GPHS.append([GPHS, protein])
            # For the single domain hit protein no the GPHS is None;
            # then the single hit is used to discriminate
            elif GPHS == None:
                hits = self.db_connector.get_all_hits_of_protein(protein)
                # Just a as control
                if len(hits) > 1:
                    print 'Something is fishy here. The protein "%s" should ' \
                          'have a GPHS!' % protein
                    continue
                elif len(hits) == 0:
                    print 'Something is fishy here. The protein "%s" should ' \
                          'have a hit!' % protein
                    continue
                annnotation = self.db_connector.get_tree_based_annotation_of_hit(
                    hits[0])
                if annnotation == 'PKS':
                    self.single_hit_PKS_proteins.append(protein)
        self.mult_hit_proteins_sorted_by_GPHS.sort()
        self.mult_hit_proteins_sorted_by_GPHS.reverse()

    def create_output_file(self):
        output_file_name = 'PKS_db_protein_overview.csv'
        output_file = open(output_file_name, 'w')
        output_file.write(
            'Protein name\tGPHS\tdomain structure\tannotation string\t'
            'annotation group\tspecies\ttaxonomy\tsource\n\n')
        output_file.write('Mutli hit proteins\n')
        for GPHS, protein in self.mult_hit_proteins_sorted_by_GPHS:
            self._write_protein_string(protein, GPHS, output_file)
        output_file.write('\n\n\nSingle hit proteins\n')
        for protein in self.single_hit_PKS_proteins:
            GPHS = '-'
            self._write_protein_string(protein, GPHS, output_file)
        output_file.close()

    def _write_protein_string(self, protein, GPHS, output_file):
        species = self.db_connector.get_species_of_protein(protein)
        taxonomy = self.db_connector.get_full_tax_of_species(species)
        output_file.write(
            '%s\t\'%s\'\t%s\t%s\t%s\t%s\t%s\t%s\n' % (
            protein,
            GPHS,
            self.db_connector.build_domain_composition_string(protein),
            self.db_connector.get_annotation_string_of_protein(protein),
            self.db_connector.get_text_string_annotation_group_of_protein(
            protein),
            species,
            taxonomy,
            self.db_connector.get_source_of_protein(protein)))

if __name__ == '__main__': main()
