#!/usr/bin/python
"""
FUNCTION: As some programs have problems with long fasta header
          simplify_fasta_headers.py produces a fasta file with
          simple number header and a tranlation table file.

          recover_full_headers_fasta.py uses an input fasta with these
          simplified headers and the translation table an builds a
          full header fasta file. 

USAGE: recover_full_headers_fasta.py <INPUT_FASTA_FILE> <TRANSLATION_TABLE_FILE>

Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2007 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys
import fastaparser

def main():
    if not len(sys.argv) == 3:
        print __doc__
        sys.exit(2)

    recovery = FullHeaderRecovery()
    recovery.set_input_fasta_file_name(sys.argv[1])
    recovery.set_input_translation_table_file_name(sys.argv[2])
    recovery.build_recoverd_header_fasta()

class FullHeaderRecovery:
    short_to_long_headers = {}
    def set_input_fasta_file_name(self, input_fasta_file_name):
        self.input_fasta_file_name = input_fasta_file_name
    
    def set_input_translation_table_file_name(
        self, input_translation_table_file_name):
        self.input_translation_table_file_name = \
                                               input_translation_table_file_name

    def build_recoverd_header_fasta(self):
        self._read_translation_table()
        self._write_full_header_fasta_file()

    def _read_translation_table(self):
        for line in open(self.input_translation_table_file_name):
            if line[0] in ['\n', '#']: continue
            split_line = line[:-1].split('\t')
            self.short_to_long_headers[split_line[0]] = split_line[1]

    def _write_full_header_fasta_file(self):
        parser = fastaparser.FastaParser()
        output_fasta_file = open('%s.header_recov' % self.input_fasta_file_name, 'w')
        for header, seq in parser.parse_fasta_file(sys.argv[1]):
            header_id = header.split('/')[0]
            old_header = self.short_to_long_headers[header_id]
            output_fasta_file.write('>%s\n%s\n' % (
                old_header, parser.shape_sequence(seq, 60)))
        output_fasta_file.close()
    
main()
